#!/usr/bin/perl
#
# Copyright (c)2010-2011 Roman Bartke.  All rights reserved.
# Author: Dipl.-Inform. Roman Bartke
#
# Created: 2010-05-29
# Updated: 2011-11-20
#

# ---------------------------------------------------------------------------
# About working with Unicode files in perl
# please note the info text below.
# ---------------------------------------------------------------------------

use Encode;

# ---------------------------------------------------------------------------
my $petme64root       = "D:/storage/project/c64/petme64/";

# pdf metadata
my $title             = ""; # test.prg
my $creator           = ""; # CBM
my $description       = ""; # Floppy Test Program
my $creatorTool       = "PetMe64";

# filenames
my $inFile            = "";
my $outFile           = "prg.fo"; 
my $foTemplateFile    = $petme64root."tpl/petme-tpl.fo";
my $blockTemplateFile = $petme64root."tpl/petmeblk-tpl.fo";

# ---------------------------------------------------------------------------


my $lower = 0;

my($result, %opt) = Getopts('?hi:o:m:b:t:n:c:d:');
$argcnt = $#ARGV + 1; 
if ((!$result) || defined($opt{'h'}) || defined($opt{'?'}) || ($argcnt != 0))
{
	print STDERR "usage: $0 -i inputFile [-o outputFile] [-t ...] ...\n";
	print STDERR "           -i: input file\n";
	print STDERR "           -o: output file\n";
	print STDERR "           -t: title\n";
	print STDERR "           -n: caption/name\n";
	print STDERR "           -c: creator\n";
	print STDERR "           -d: description\n";
	print STDERR "           -m: main fo template\n";
	print STDERR "           -b: block sub template\n";
  	exit;
}
 
if (!defined($opt{'i'}))
{
	die "No input file given";
}

$inFile            = $opt{'i'};
$outFile           = $opt{'o'};

$caption           = $opt{'n'} if defined($opt{'n'});
$title             = $opt{'t'} if defined($opt{'t'});
$creator           = $opt{'c'} if defined($opt{'c'});
$description       = $opt{'d'} if defined($opt{'d'});

$foTemplateFile    = $opt{'m'} if defined($opt{'m'});
$blockTemplateFile = $opt{'b'} if defined($opt{'b'});

print "infile:           ",$inFile,"\n";
print "outfile:          ",$outFile,"\n";
print "title:            ",$title,"\n";
print "caption:          ",$caption,"\n";
print "creator:          ",$creator,"\n";
print "description:      ",$description,"\n";
print "main fo template: ",$foTemplateFile,"\n";
print "block template:   ",$blockTemplateFile,"\n";

# ---------------------------------------------------------------------------

my $foTemplate        = FileToString($foTemplateFile);
my $blockTemplate     = FileToString($blockTemplateFile);

open(IN, "<:encoding(UTF-16)", $inFile) 
	|| die "Cannot open input file $inFile: $!";

open( OUT, ">:raw:encoding(UTF16-LE):utf8", "$outFile" ) 
	|| die "Cannot open output file $outFile: $!";
print OUT "\x{FEFF}";  # print BOM (Byte Order Mark) for the unicode file

my $string = "";
my $blocks = "";
my $fo = $foTemplate;

while( $string = <IN> ) 
{
	chomp $string;
	# replacements of <, > and & into &lt; , $gt; and $amp; are
	# not needed because the code points lie at U+e1xx
	my $block = $blockTemplate;
	$block =~ s/{petsciiLine}/$string/;
#	$string = encode('UTF-16', $string);
	$blocks .= $block;
}
close(IN);

$fo =~ s/{title}/$title/;
$fo =~ s/{caption}/$caption/;
$fo =~ s/{creator}/$creator/;
$fo =~ s/{description}/$description/;
$fo =~ s/{creatorTool}/$creatorTool/;
$fo =~ s/{blocks}/$blocks/;

print OUT $fo;

close(OUT);

sub FileToString {
	my $tplFileName = shift;
	open(FILE, "<:encoding(UTF-16)", $tplFileName) 
		|| die "Cannot open template file $tplFileName: $!";
	my $inFile = "";
	while (<FILE>) {
		$inFile .= $_;
	}
	close (FILE);
	return $inFile;
}

#
# Writing a Unicode file via perl ...
#
# Source: http://blogs.msdn.com/b/brettsh/archive/2006/06/07/620986.aspx?PageIndex=2
#
# Some notes from someone smarter than me about Perl and Unicode ...
# (I had to read it twice to really grok how the ":raw" type parts were
# like piping through converting text commands, but in reverse so you 
# read them right to left)
# ----
#
# Which encoding do you want to use? UTF16-LE is the standard on Windows (nearly
# all characters are encoded as 2 bytes), UTF8 is the standard everywhere else 
# (characters are variable length and all ASCII characters are a single byte).
#
# Here's what I've figured out after lots of experimentation. To get UTF16-LE 
# output you need to play a few games with perl...
#
#   open my $FH, ">:raw:encoding(UTF16-LE):crlf:utf8", "e:\\test.txt";
#   print $FH "\x{FEFF}";
#   print $FH "hello unicode world!\nThis is a test.\n";
#   close $FH;
# Reading the IO layers from right to left (the order that they will be applied 
# as they pass from perl to the file) ...
#
# Apply the :utf8 layer first. This doesn't do much except tell perl that we're 
# going to pass "characters" to this file handle instead of bytes so that it 
# doesn't give us "Wide character in print ..." warnings.
#
# Next, apply the :crlf layer as text goes from perl out to the file. This 
# transforms \n (0x0A) into \r\n (0x0D 0x0A) giving you DOS line endings. Perl 
# normally applies this by default on Windows but it would do it at the wrong 
# stage of the pipeline so we removed it (see below), this is where it ought to 
# be.
#
# Next apply the UTF16-LE (little endian) encoding. This takes the characters 
# and transforms them to that encoding. So 0x0A turns into 0x0A 0x00. Note that 
# if you just say 'UTF16' the default endianness is big endian which is 
# backwards from how Windows likes it. However, because we're explicitly 
# specifiying the endianness perl will not write a BOM (byte order mark) at the 
# beginning of the file. We have to make up for that later.
#
# Finally, the :raw psuedo layer just removes the default (on Windows) :crlf 
# layer that transforms \n into \r\n for DOS style line endings. This is 
# necessary because otherwise it would be applied at the wrong place in the 
# pipeline. Without this the encoding layer would turn 0x0A into 0x0A 0x00 and 
# then the crlf layer would turn that into 0x0D 0x0A 0x0A and that's just goofy.
#
# Now that we've got the file opened with the right IO layers in place we can 
# almost write to it. First we need to manually write the BOM that will tell 
# readers of this file what endianness it is in. That's what the 
# print $FH "\x{FEFF}" does.
#
# Finally we can just print text out.
#
# If you want UTF8, I'm pretty sure it's a lot easier. Also, this is also a lot 
# easier on unix, the CRLF ordering problem is definitely a bug but the default 
# to big endian (and ensuing games to get the BOM to output without a warning) 
# are by design.
# I'm pretty sure that none of the core perl maintainers use perl 
# on Windows (even though at least one keeps perl on VMS working...).
#


sub Getopts 
{
  my($argumentative) = @_;
  my(@args,$first,$rest);
  my($errs) = 0;
  my(%opt) = ();
  local($[) = 0;
  local($_);
  
  @args = split( / */, $argumentative );
  while(@ARGV && ($_ = $ARGV[0]) =~ /^-(.)(.*)/) {
    ($first,$rest) = ($1,$2);
    my $pos = index($argumentative,$first);
    if($pos >= $[) {
      if($args[$pos+1] eq ':') {
	shift(@ARGV);
	if($rest eq '') {
	  ++$errs unless @ARGV;
	  $rest = shift(@ARGV);
	}
	$opt{$first} = $rest;
      }
      else {
	$opt{$first} = 1;
	if($rest eq '') {
	  shift(@ARGV);
	}
	else {
	  $ARGV[0] = "-$rest";
	}
      }
    }
    else {
      print STDERR "Unknown option: $first\n";
      ++$errs;
      if($rest ne '') {
	$ARGV[0] = "-$rest";
      }
      else {
	shift(@ARGV);
      }
    }
  }
  return($errs == 0, %opt);
}
