.ifdef SuperTape
;.include "lisp128st.s"
;
;
; Supertape (Input64 Diskmagazine)
; ================================
; info by Andreas Matthies <andreas.matthies@arcormail.de>
;
;
; Encoding:
; ---------
;
; Three different pulses are used (not 100% accurate)
;
; Short  (s):    464 T-States ~ 261 cycles of C64(PAL)
; Middle (m):    703 T-States ~ 396 cycles of C64(PAL)
; Long   (l):    945 T-States ~ 532 cycles of C64(PAL)
;
; A pulse s represents a Bit0.
; A pulse m first represents Bit1Bit0 (two Bits) then Bit1(one Bit), then again
; Bit1Bit0 and so on.
; A pulse l represents Bit1Bit1 (two Bits) and resets the counter for the m
; pulses, means that the next m following an l is always Bit1Bit0.
;
; Each Data Byte has 8 Bits with least significant Bit first.
;
; Lead-In Byte is $16 and Sync Byte is $3A (or $2A; the byte is masked with $EF
; and compared to $2A)
; for the header-block and $C5 for the data-block.
;
; Structure:
; ---------
;
; Each file consists of two blocks, the header and the data. Each block follows a
; lead-in, which is at least 11 bytes of $16. The structure of the header is:
;
; $00 - $0F: filename
; $10 - $10: #00 (not used?)
; $11 - $12: start-adress
; $13 - $14: length of file
; $15 - $18: #00 (not used?)
; $19 - $1A: parity-checksum, should match the amount of Bits1 in $00-$18.
;
; The strucure of the data-block is even more simple:
;
; $0000 - ...: data
; followed by two bytes parity-checksum, that should match the amount of Bits1 in
; data modulo $10000.
; =====================================
InitSuperTape:
        LDA #$96
        STA byte_FE
        LDA #<ICLOSE
        STA lICLOSE
        LDA #>ICLOSE
        STA lICLOSE+1
        LDA #<IOPEN
        STA lIOPEN
        LDA #>IOPEN
        STA lIOPEN+1
        LDA #<IGETIN
        STA lIGETIN
        LDA #>IGETIN
        STA lIGETIN+1
        LDA #<ICHKIN
        STA lICHKIN
        LDA #>ICHKIN
        STA lICHKIN+1
        LDA #<ICKOUT
        STA lICKOUT
        LDA #>ICKOUT
        STA lICKOUT+1
        LDA #<IBASIN
        STA lIBASIN
        LDA #>IBASIN
        STA lIBASIN+1
        LDA #<IBSOUT
        STA lIBSOUT
        LDA #>IBSOUT
        STA lIBSOUT+1
        RTS                            ; PC <- (Stack)
; -------------------------------------
aWrongTapeHeade:.BYTE "WRONG TAPE HEADER-",$D,0
aSearchingConti:.BYTE "SEARCHING CONTINUED",$D,$D,0
; -------------------------------------
IOPEN:
        LDX zLA
        BNE IOPEN1                     ; if Z=0, PC = PC + offset
        JMP k_IO_ERROR_6_NotInputFile
; -------------------------------------
IOPEN1:
        JSR kFindFileInLogicalFileTable; Find the File in the Logical File Table
        BNE IOPEN2                     ; if Z=0, PC = PC + offset
        JMP k_IO_ERROR_2_FileOpen
; -------------------------------------
IOPEN2:
        LDX zLDTND
        CPX #$A                        ; (X - M) -> NZC
        BCC IOPEN3                     ; if C=0, PC = PC + offset
        JMP k_IOERROR_1_TooManyFiles
; -------------------------------------
IOPEN3:
        INC zLDTND
        LDA zLA
        STA lLAT,X
        LDA zSA
        ORA #$60
        STA zSA
        STA lSAT,X
        LDA zFA                        ; Device Number
        STA lFAT,X
; =====================================
IOPEN4:
        CMP #7                         ; (A - M) -> NZC
        BEQ IOPEN5_SuperTape           ; if Z=1, PC = PC + offset
        JMP OpenEntry
; -------------------------------------
IOPEN5_SuperTape:
        LDA zSA
        AND #$F
        BEQ IOPEN6                     ; if Z=1, PC = PC + offset
        JSR BeginTapeWrite
        JSR CheckFileName
        LDA zSA
        AND #$F
        STA Tape_SA
        LDA #<lTBUFFR
        STA TBUFFR_Ptr
        LDA #>lTBUFFR
        STA TBUFFR_Ptr+1
        JSR ClearBuff_0351_to_0354
        STA ClearByte_0350
        STA zBUFPNT
        LDA #$19
        STA byte_34F
        LDA #$55
        STA zVARPNT
        JSR sub_2D1E
        LDA #$2A
        JSR sub_2C5A
        JMP EndOfTapeIO
; -------------------------------------
IOPEN6:
        JSR LoadInit
        JSR sub_2A21
        JSR sub_2B94
        BCC loc_28D8                   ; if C=0, PC = PC + offset
        LDY #$28                       ; MSG: WrongTapeHeader
        JSR MsgB
        LDY #$3C
        JSR MsgB                       ; MSG: SearchingContinued
        JSR EndOfTapeRead
        JMP IOPEN6
; =====================================
sub_28D5:
        JSR LoadInit
loc_28D8:
        LDA #<lTBUFFR
        STA zVARNAM
        LDA #>lTBUFFR
        STA zVARNAM+1
        LDA #$FC
        STA zEAL
        LDA #3
        STA zEAL+1
        LDA #$96
        STA byte_FD
        JSR CheckSumBuf
        BCS Msg_WrongTapeHeader        ; if C=1, PC = PC + offset
        LDA #0
        STA zBUFPNT
        JMP EndOfTapeRead
; -------------------------------------
Msg_WrongTapeHeader:
        LDY #$28                       ; '('      ; MSG: WrongTapeHeader
        JSR MsgB
        JSR EndOfTapeRead
        JMP sub_28D5
; -------------------------------------
ICLOSE:
        JSR kEntryFindLogFileNo
        BEQ ICLOSE1                    ; if Z=1, PC = PC + offset
        CLC                            ; C <- 0
        RTS                            ; PC <- (Stack)
; -------------------------------------
ICLOSE1:
        JSR SetFileParams
        TXA
        PHA                            ; Stack <- (A)
        LDA zFA                        ; Device Number
        CMP #7                         ; (A - M) -> NZC
        BEQ ICLOSE2                    ; if Z=1, PC = PC + offset
        JMP kCloseEntry
; -------------------------------------
ICLOSE2:
        LDA zSA
        AND #1
        BEQ ICLOSE3                    ; if Z=1, PC = PC + offset
        JSR kGetTBUFFPtr2YX
        LDA #$FC
        STA zVARPNT
        LDA #$D6
        STA byte_FE
        LDA zBUFPNT
        STA lTBUFFR
        JSR sub_2D3E
ICLOSE3:
        JMP kEntryCloseTapeFile
; -------------------------------------
IBSOUT:
        PHA                            ; Stack <- (A)
        LDA zDFLTO
        CMP #7                         ; (A - M) -> NZC
        BEQ IBSOUT1                    ; if Z=1, PC = PC + offset
        PLA
        JMP unk_F1CA
; -------------------------------------
IBSOUT1:
        PLA
        STA zPTR1
        TXA
        PHA                            ; Stack <- (A)
        TYA
        PHA                            ; Stack <- (A)
        JSR Inc_TBUFF_Ptr
        BNE IBSOUT2                    ; if Z=0, PC = PC + offset
        LDA #$FC
        STA zVARPNT
        LDA #$96
        STA byte_FE
        JSR sub_2D3E
        JMP unk_F1F1
; -------------------------------------
IBSOUT2:
        JMP unk_F1F8
; -------------------------------------
IBASIN:
        LDA zDFLTN
        CMP #7                         ; (A - M) -> NZC
        BEQ IBASIN1                    ; if Z=1, PC = PC + offset
        JMP kernalBASIN
; -------------------------------------
IBASIN1:
        STX zTEMPX
IBASIN2:
        JSR Inc_TBUFF_Ptr
        BNE IBASIN5                    ; if Z=0, PC = PC + offset
        BEQ IBASIN4                    ; if Z=1, PC = PC + offset
IBASIN3:
        LDY #$28                       ; MSG: WrongTapeHeader
        JSR MsgB
        LDY #$3C                       ; MSG: SearchingContiued
        JSR MsgB
IBASIN4:
        JSR sub_28D5
        BCS IBASIN3                    ; if C=1, PC = PC + offset
        LDA #0
        STA zBUFPNT
        BEQ IBASIN2                    ; if Z=1, PC = PC + offset
IBASIN5:
        NOP                            ; [no operation]
        DEC lTBUFFR
        BNE IBASIN6                    ; if Z=0, PC = PC + offset
        LDA #$40
        JSR kSetStatus
        LDA #$D
; -------------------------------------
        .BYTE $2C                      ; BIT opcode to mask next 2 bytes
; -------------------------------------
IBASIN6:
        LDA (zTAPE1),Y
        CLC                            ; C <- 0
        LDX zTEMPX
        RTS                            ; PC <- (Stack)
; -------------------------------------
ICKOUT:
        JSR kFindFileInLogicalFileTable
        BEQ ICKOUT1                    ; if Z=1, PC = PC + offset
        JMP k_IO_ERROR_3_FileNotOpen
; -------------------------------------
ICKOUT1:
        JSR SetFileParams
        LDA zFA                        ; Device Number
        CMP #7                         ; (A - M) -> NZC
        BEQ ICKOUT2                    ; if Z=1, PC = PC + offset
        JMP unk_F25B
; -------------------------------------
ICKOUT2:
        JMP unk_F26F
; -------------------------------------
ICHKIN:
        JSR kFindFileInLogicalFileTable
        BEQ ICHKIN1                    ; if Z=1, PC = PC + offset
        JMP k_IO_ERROR_3_FileNotOpen
; -------------------------------------
ICHKIN1:
        JSR SetFileParams
        LDA zFA                        ; Device Number
        CMP #7                         ; (A - M) -> NZC
        BEQ ICHKIN2                    ; if Z=1, PC = PC + offset
        JMP unk_F219
; -------------------------------------
ICHKIN2:
        JMP unk_F22A
; -------------------------------------
JmpVec2: .WORD kernalGETIN
; -------------------------------------
IGETIN:
        LDA zDFLTN
        CMP #7                         ; (A - M) -> NZC
        BEQ IGETIN2                    ; if Z=1, PC = PC + offset
        JMP (JmpVec2)
; -------------------------------------
IGETIN2:
        JMP kCHRIN
; =====================================
LoadInit:
        SEI                            ; I <- 1
        JSR WaitForPressPlayOnTape
        LDA zR6510
        AND #$1F
        STA zR6510
        STA zCAS1
        LDA #<hNMI_B
        STA lNMINV
        LDA #>hNMI_B
        STA lNMINV+1
        LDA lCINV
        STA lIRQTMP
        LDA lCINV+1
        STA lIRQTMP+1
        LDA #<hIRQ
        STA lCINV
        LDA #>hIRQ
        STA lCINV+1
        LDA #$7F
        STA CIA1ICR
        LDA #$90
        STA CIA1ICR
        LDA VIC_D011
        AND #$EF                       ; ''      ; SCREEN OFF
        STA VIC_D011
        LDA #$46
        STA CIA1TALO
        LDA #$81
        STA CIA1TAHI
        LDA #$73
        STA zOLDTXT
        CLI                            ; I <- 0
        RTS                            ; PC <- (Stack)
; =====================================
sub_2A21:
        JSR sub_2B3F
        CMP #$2A                       ; '*'      ; (A - M) -> NZC
        BNE sub_2A21                   ; if Z=0, PC = PC + offset
        STY CheckSum
        STY CheckSum+1
loc_2A2C:
        BIT zINPPTR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL loc_2A2C                   ; if N=0, PC = PC + offset
        STY zINPPTR
        LDA byte_40
        STA lTBUFFR,Y
        SEC                            ; C <- 1
        ROR A                          ; C<-A0 & A<- (A7=C + A>>1)
loc_2A39:
        BCC loc_2A41                   ; if C=0, PC = PC + offset
        INC CheckSum
        BNE loc_2A41                   ; if Z=0, PC = PC + offset
        INC CheckSum+1
loc_2A41:
        LSR A
        BNE loc_2A39                   ; if Z=0, PC = PC + offset
        INY
        CPY #$19                       ; (Y - M) -> NZC
        BCC loc_2A2C                   ; if C=0, PC = PC + offset
        JSR sub_2B36
        CMP CheckSum                   ; (A - M) -> NZC
        BNE loc_2A8B                   ; if Z=0, PC = PC + offset
        JSR sub_2B36
        CMP CheckSum+1                 ; (A - M) -> NZC
        BNE loc_2A8B                   ; if Z=0, PC = PC + offset
        LDY #0
        CPY zFNLEN                     ; (Y - M) -> NZC
        BEQ loc_2A8D                   ; if Z=1, PC = PC + offset
        STY zBSOUR
        STY byte_3C
        LDX #0
loc_2A63:
        LDA (zFNADR),Y
        CMP #'.'                       ; (A - M) -> NZC
        BEQ loc_2A84                   ; if Z=1, PC = PC + offset
        CMP #'*'                       ; (A - M) -> NZC
        BEQ loc_2A92                   ; if Z=1, PC = PC + offset
        LDY zBSOUR
        CMP #'?'                       ; (A - M) -> NZC
        BEQ loc_2A78                   ; if Z=1, PC = PC + offset
        CMP lTBUFFR,Y                  ; (A - M) -> NZC
        BNE loc_2A9B                   ; if Z=0, PC = PC + offset
loc_2A78:
        INY
loc_2A79:
        INX
        CPX zFNLEN                     ; (X - M) -> NZC
        BCS loc_2A8D                   ; if C=1, PC = PC + offset
        STY zBSOUR
        TXA
        TAY
        BCC loc_2A63                   ; if C=0, PC = PC + offset
loc_2A84:
        DEC byte_3C
        LDY #$D
        JMP loc_2A79
; -------------------------------------
loc_2A8B:
        SEC                            ; C <- 1
        RTS                            ; PC <- (Stack)
; -------------------------------------
loc_2A8D:
        JSR ShowFoundFileName
        CLC                            ; C <- 0
        RTS                            ; PC <- (Stack)
; -------------------------------------
loc_2A92:
        BIT byte_3C                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BMI loc_2A8D                   ; if N=1, PC = PC + offset
        LDY #$C
        JMP loc_2A79
; -------------------------------------
loc_2A9B:
        JSR EndOfTapeRead
        JSR ShowFoundFileName
        LDA #3
        STA zVERCKK
loc_2AA5:
        JSR TapePause
        DEC zVERCKK
        BNE loc_2AA5                   ; if Z=0, PC = PC + offset
        JSR LoadInit
        JMP sub_2A21
; =====================================
TapePause:
        LDX #0
        LDY #0
TapePause1:
        DEX
        BNE TapePause1                 ; if Z=0, PC = PC + offset
        DEY
        BNE TapePause1                 ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
FileNameInvalidError:
        LDY #0
FileNameInvalidError_1:
        LDA aFilnameInv,Y              ; "? FILENAME INVALID\r"
        INY
        JSR kCHROUT
        CMP #0                         ; (A - M) -> NZC
        BNE FileNameInvalidError_1     ; if Z=0, PC = PC + offset
        PLA
        PLA
        RTS                            ; PC <- (Stack)
; -------------------------------------
aFilnameInv: .BYTE "? FILENAME INVALID",$D,0
; -------------------------------------
hIRQ:
        LDA CIA1ICR
        LDA #$D9
        LDX CIA1TALO
        LDY CIA1TAHI
        STA CIA1CRA
        BMI hIRQ4                      ; if N=1, PC = PC + offset
        SEC                            ; C <- 1
        ROR CurrentByte                ; C<-A0 & A<- (A7=C + A>>1)
        BIT zDATLIN                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BMI hIRQ1                      ; if N=1, PC = PC + offset
        BCC hIRQ1                      ; if C=0, PC = PC + offset
        LDA CurrentByte
        STA byte_40
        LDA #$80
        STA CurrentByte
        STA zINPPTR
hIRQ1:
        CPY #$79                       ; 'y'      ; (Y - M) -> NZC
        BCC hIRQ2                      ; if C=0, PC = PC + offset
        CPX zOLDTXT                    ; (X - M) -> NZC
        BCS hIRQ3                      ; if C=1, PC = PC + offset
hIRQ2:
        LDA #0
        STA byte_42
        SEC                            ; C <- 1
        BCS hIRQ5                      ; if C=1, PC = PC + offset
hIRQ3:
        LDA byte_42
        EOR #$80
        STA byte_42
        BIT byte_42                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL hIRQ6                      ; if N=0, PC = PC + offset
hIRQ4:
        CLC                            ; C <- 0
hIRQ5:
        ROR CurrentByte                ; C<-A0 & A<- (A7=C + A>>1)
        BIT zDATLIN                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BMI hIRQ6                      ; if N=1, PC = PC + offset
        BCC hIRQ6                      ; if C=0, PC = PC + offset
        LDA CurrentByte
        STA byte_40
        LDA #$80
        STA CurrentByte
        STA zINPPTR
hIRQ6:
        PLA
        TAY
        PLA
        TAX
        PLA
        RTI                            ; P <- (Stack), PC <-(Stack)
; =====================================
sub_2B36:
        BIT zINPPTR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL sub_2B36                   ; if N=0, PC = PC + offset
        STY zINPPTR
        LDA byte_40
        RTS                            ; PC <- (Stack)
; =====================================
sub_2B3F:
        LDY #$80
        STY zDATLIN
loc_2B43:
        LDA CurrentByte
        CMP #$16                       ; (A - M) -> NZC
        BNE loc_2B43                   ; if Z=0, PC = PC + offset
        STY CurrentByte
        LDY #$A
        STY zDATLIN
        STY zINPPTR
loc_2B51:
        BIT zINPPTR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL loc_2B51                   ; if N=0, PC = PC + offset
        STY zINPPTR
        LDA byte_40
        CMP #$16                       ; (A - M) -> NZC
        BNE sub_2B3F                   ; if Z=0, PC = PC + offset
        DEY
        BNE loc_2B51                   ; if Z=0, PC = PC + offset
loc_2B60:
        JSR sub_2B36
        CMP #$16                       ; (A - M) -> NZC
        BEQ loc_2B60                   ; if Z=1, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
EndOfTapeIO:
        JSR EndOfTapeIRQ
        LDA #<hNMI
        STA lNMINV
        LDA #>hNMI
        STA lNMINV+1
        CLC                            ; C <- 0
        CLI                            ; I <- 0
        RTS                            ; PC <- (Stack)
; -------------------------------------
hNMI_B:
        JSR EndOfTapeIO
        JSR IOINIT
        JMP hNMI
; -------------------------------------
ShowFileName:
        LDY #0
ShowFileName1:
        LDA lTBUFFR,Y
        JSR kCHROUT
        INY
        CPY #$10                       ; (Y - M) -> NZC
        BCC ShowFileName1              ; if C=0, PC = PC + offset
        LDA #$D
        JSR kCHROUT
        RTS                            ; PC <- (Stack)
; =====================================
sub_2B94:
        BIT Tape_SA                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL locret_2BA7                ; if N=0, PC = PC + offset
        LDA #$9C
        STA CIA1TALO
        LDA #$80
        STA CIA1TAHI
        LDA #$BB
        STA zOLDTXT
locret_2BA7:
        RTS                            ; PC <- (Stack)
; =====================================
CheckSumBuf:
        JSR sub_2B3F
        CMP byte_FD                    ; (A - M) -> NZC
        BEQ CheckSumBuf1               ; if Z=1, PC = PC + offset
        AND #$BF
        CMP byte_FD                    ; (A - M) -> NZC
        BNE CheckSumBuf7               ; if Z=0, PC = PC + offset
        LDA #$80
        JSR kSetStatus
CheckSumBuf1:
        STY CheckSum
        STY CheckSum+1
CheckSumBuf2:
        BIT zINPPTR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL CheckSumBuf2               ; if N=0, PC = PC + offset
        STY zINPPTR
        LDA byte_40
        STA (zVARNAM),Y
        SEC                            ; C <- 1
        ROR A                          ; C<-A0 & A<- (A7=C + A>>1)
CheckSumBuf3:
        BCC CheckSumBuf4               ; if C=0, PC = PC + offset
        INC CheckSum
        BNE CheckSumBuf4               ; if Z=0, PC = PC + offset
        INC CheckSum+1
CheckSumBuf4:
        LSR A
        BNE CheckSumBuf3               ; if Z=0, PC = PC + offset
        INC zVARNAM
        BNE CheckSumBuf5               ; if Z=0, PC = PC + offset
        INC zVARNAM+1
CheckSumBuf5:
        LDA zVARNAM
        CMP zEAL                       ; (A - M) -> NZC
        BNE CheckSumBuf2               ; if Z=0, PC = PC + offset
        LDA zVARNAM+1
        CMP zEAL+1                     ; (A - M) -> NZC
        BNE CheckSumBuf2               ; if Z=0, PC = PC + offset
CheckSumBuf6:
        BIT zINPPTR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL CheckSumBuf6               ; if N=0, PC = PC + offset
        STY zINPPTR
        LDA byte_40
        CMP CheckSum                   ; (A - M) -> NZC
CheckSumBuf7:
        BNE CheckSumBuf9               ; if Z=0, PC = PC + offset
CheckSumBuf8:
        BIT zINPPTR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BPL CheckSumBuf8               ; if N=0, PC = PC + offset
        STY zINPPTR
        LDA byte_40
        CMP CheckSum+1                 ; (A - M) -> NZC
        BNE CheckSumBuf9               ; if Z=0, PC = PC + offset
        CLC                            ; C <- 0
        RTS                            ; PC <- (Stack)
; -------------------------------------
CheckSumBuf9:
        SEC                            ; C <- 1
        RTS                            ; PC <- (Stack)
; =====================================
EndOfTapeRead:
        JSR EndOfTapeIO
        LDX zEAL
        LDY zEAL+1
        LDA #0
        CLC                            ; C <- 0
        RTS                            ; PC <- (Stack)
; =====================================
sub_2C0E:
        STA CurrentByte
; =====================================
WBYTE:
        LDY #4
WLOOP:
        LSR CurrentByte
        BCC loc_2C1B                   ; if C=0, PC = PC + offset
        LDA byte_FC
        STA CIA2TALO
loc_2C1B:
        LDA #1
loc_2C1D:
        BIT CIA2ICR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BEQ loc_2C1D                   ; if Z=1, PC = PC + offset
        LDA #$D9
        STA CIA2CRA
        LDA zR6510
        EOR #8
        STA zR6510
        BCS WPRUEF                     ; if C=1, PC = PC + offset
        LDA #1
loc_2C31:
        BIT CIA2ICR                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BEQ loc_2C31                   ; if Z=1, PC = PC + offset
        LDA #$D9
        STA CIA2CRA
        LDA zR6510
        EOR #8
        STA zR6510
        DEY
        BNE WLOOP                      ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; -------------------------------------
WPRUEF:
        LDA CheckSum
        ADC #0
        STA CheckSum
        LDA CheckSum+1
        ADC #0
        STA CheckSum+1
        LDA zFREKXP
        STA CIA2TALO
        DEY
        BNE WLOOP                      ; if Z=0, PC = PC + offset
        RTS                            ; PC <- (Stack)
; =====================================
sub_2C5A:
        PHA                            ; Stack <- (A)
        LDX #$40
loc_2C5D:
        LDA #$16
        JSR sub_2C0E
        JSR WBYTE
        DEX
        BNE loc_2C5D                   ; if Z=0, PC = PC + offset
        PLA
        JSR sub_2C0E
        JSR WBYTE
        LDY #0
        STY CheckSum
        STY CheckSum+1
loc_2C75:
        LDA (zVARNAM),Y
        JSR sub_2C0E
        INC zVARNAM
        BNE loc_2C80                   ; if Z=0, PC = PC + offset
        INC zVARNAM+1
loc_2C80:
        JSR WBYTE
        LDA zVARNAM
        CMP zVARPNT                    ; (A - M) -> NZC
        BNE loc_2C75                   ; if Z=0, PC = PC + offset
        LDA zVARNAM+1
        CMP byte_48                    ; (A - M) -> NZC
        BNE loc_2C75                   ; if Z=0, PC = PC + offset
        LDX CheckSum+1
        LDA CheckSum
        JSR sub_2C0E
        JSR WBYTE
        TXA
        JSR sub_2C0E
        JSR WBYTE
        JMP sub_2C0E
; =====================================
CheckFileName:
        LDY #0
        LDA #' '
CheckFileName1:
        STA lTBUFFR,Y
        INY
        CPY #$10                       ; (Y - M) -> NZC
        BCC CheckFileName1             ; if C=0, PC = PC + offset
        LDY #0
        STY byte_3C
        STY zBSOUR
        CPY zFNLEN                     ; (Y - M) -> NZC
        BEQ CheckFileName4             ; if Z=1, PC = PC + offset
        TYA
        TAX
CheckFileName2:
        LDA (zFNADR),Y
        CMP #'.'                       ; (A - M) -> NZC
        BNE CheckFileName5             ; if Z=0, PC = PC + offset
        CPY #$D                        ; (Y - M) -> NZC
        BCS FileNameInvalid            ; if C=1, PC = PC + offset
        BIT byte_3C                    ; Z <- ~(A /\ M) N<-M7 V<-M6
        BMI FileNameInvalid            ; if N=1, PC = PC + offset
        DEC byte_3C
        LDY #$C
CheckFileName3:
        STA lTBUFFR,Y
        INY
        STY zBSOUR
        CPY #$10                       ; (Y - M) -> NZC
        BCS CheckFileName4             ; if C=1, PC = PC + offset
        INX
        CPX zFNLEN                     ; (X - M) -> NZC
        BCS CheckFileName4             ; if C=1, PC = PC + offset
        TXA
        TAY
        JMP CheckFileName2
; -------------------------------------
CheckFileName4:
        RTS                            ; PC <- (Stack)
; -------------------------------------
CheckFileName5:
        LDY zBSOUR
        JMP CheckFileName3
; -------------------------------------
FileNameInvalid:
        JMP FileNameInvalidError
; =====================================
BeginTapeWrite:
        SEI                            ; I <- 1
        JSR WaitForPressAndRecordOnTape
        LDA zR6510
        AND #$1F
        STA zR6510
        STA zCAS1
        JSR TapePause
        JSR TapePause
        LDA #<hNMI_B
        STA lNMINV
        LDA #>hNMI_B
        STA lNMINV+1
        LDA VIC_D011
        AND #$EF                       ; ''      ; screen off
        STA VIC_D011
        RTS                            ; PC <- (Stack)
; =====================================
ClearBuff_0351_to_0354:
        LDA #0
        STA byte_351
        STA byte_352
        STA byte_353
        STA byte_354
        RTS                            ; PC <- (Stack)
; =====================================
sub_2D1E:
        LDA #0
        STA CIA2TAHI
        LDA #$3C
        STA zVARNAM
        LDA #3
        STA zVARNAM+1
        STA byte_48
        LDA #$FF
        STA byte_FC
        LDA #$78
        STA zFREKXP
        STA CIA2TALO
        LDA #$D9
        STA CIA2CRA
        RTS                            ; PC <- (Stack)
; =====================================
sub_2D3E:
        JSR BeginTapeWrite
        JSR sub_2D1E
        LDA zBUFPNT
        STA lTBUFFR
        LDA byte_FE
        JSR sub_2C5A
        JMP EndOfTapeIO
; =====================================
ShowFoundFileName:
        LDY #$63                       ; offset of kernal's "Found" message
        JSR kSystemMessage
        JMP ShowFileName
; =====================================
.endif
